@@ -1,7 +1,6 @@
 var API3 = function(m)
 {
 
-
 /**
  * Provides an API for the rest of the AI scripts to query the world state at a
  * higher level than the raw data.
@@ -20,49 +19,53 @@ m.GameState.prototype.init = function(SharedScript, state, player) {
 	this.entities = SharedScript.entities;
 	this.player = player;
 	this.playerData = SharedScript.playersData[this.player];
-	this.techModifications = SharedScript._techModifications[this.player];
-	this.barterPrices = SharedScript.barterPrices;
 	this.gameType = SharedScript.gameType;
+	this.alliedVictory = SharedScript.alliedVictory;
+	this.ceasefireActive = SharedScript.ceasefireActive;
 
 	// get the list of possible phases for this civ:
 	// we assume all of them are researchable from the civil centre
-	this.phases = [ { name: "phase_village" }, { name: "phase_town" }, { name: "phase_city" } ];
+	this.phases = [{ name: "phase_village" }, { name: "phase_town" }, { name: "phase_city" }];
 	let cctemplate = this.getTemplate(this.applyCiv("structures/{civ}_civil_centre"));
 	if (!cctemplate)
 		return;
-	let techs = cctemplate.researchableTechs(this.civ());
-	for (let i = 0; i < this.phases.length; ++i)
+	let civ = this.getPlayerCiv();
+	let techs = cctemplate.researchableTechs(civ);
+	for (let phase of this.phases)
 	{
-		let k = techs.indexOf(this.phases[i].name);
-		if (k != -1)
+		phase.requirements = [];
+		let k = techs.indexOf(phase.name);
+		if (k !== -1)
 		{
-			this.phases[i].requirements = (this.getTemplate(techs[k]))._template.requirements;
-			continue;
+			let reqs = DeriveTechnologyRequirements(this.getTemplate(techs[k])._template, civ);
+			if (reqs)
+			{
+				phase.requirements = reqs;
+				continue;
+			}
 		}
 		for (let tech of techs)
 		{
 			let template = (this.getTemplate(tech))._template;
-			if (template.replaces && template.replaces.indexOf(this.phases[i].name) != -1)
+			if (template.replaces && template.replaces.indexOf(phase.name) != -1)
 			{
-				this.phases[i].name = tech;
-				this.phases[i].requirements = template.requirements;
-				break;
+				let reqs = DeriveTechnologyRequirements(template, civ);
+				if (reqs)
+				{
+					phase.name = tech;
+					phase.requirements = reqs;
+					break;
+				}
 			}
 		}
 	}
 };
 
-m.GameState.prototype.update = function(SharedScript, state) {
-	this.sharedScript = SharedScript;
-	this.EntCollecNames = SharedScript._entityCollectionsName;
+m.GameState.prototype.update = function(SharedScript)
+{
 	this.timeElapsed = SharedScript.timeElapsed;
-	this.templates = SharedScript._templates;
-	this.techTemplates = SharedScript._techTemplates;
-	this._entities = SharedScript._entities;
-	this.entities = SharedScript.entities;
 	this.playerData = SharedScript.playersData[this.player];
-	this.techModifications = SharedScript._techModifications[this.player];
-	this.barterPrices = SharedScript.barterPrices;
+	this.ceasefireActive = SharedScript.ceasefireActive;
 };
 
 m.GameState.prototype.updatingCollection = function(id, filter, collection)
@@ -74,7 +77,7 @@ m.GameState.prototype.updatingCollection = function(id, filter, collection)
 m.GameState.prototype.destroyCollection = function(id)
 {
 	let gid = this.player + "-" + id;	// automatically add the player ID
-	return this.destroyGlobalCollection(gid);
+	this.destroyGlobalCollection(gid);
 };
 
 m.GameState.prototype.getEC = function(id)
@@ -88,12 +91,9 @@ m.GameState.prototype.updatingGlobalCollection = function(id, filter, collection
 	if (this.EntCollecNames.has(id))
 		return this.EntCollecNames.get(id);
 
-	if (collection !== undefined)
-		var newCollection = collection.filter(filter);
-	else
-		var newCollection = this.entities.filter(filter);
+	let newCollection = collection !== undefined ? collection.filter(filter) : this.entities.filter(filter);
 	newCollection.registerUpdates();
-	this.EntCollecNames.set(id, newCollection);	
+	this.EntCollecNames.set(id, newCollection);
 	return newCollection;
 };
 
@@ -120,7 +120,7 @@ m.GameState.prototype.getTimeElapsed = function()
 
 m.GameState.prototype.getBarterPrices = function()
 {
-	return this.barterPrices;
+	return this.playerData.barterPrices;
 };
 
 m.GameState.prototype.getGameType = function()
@@ -128,23 +128,35 @@ m.GameState.prototype.getGameType = function()
 	return this.gameType;
 };
 
+m.GameState.prototype.getAlliedVictory = function()
+{
+	return this.alliedVictory;
+};
+
+m.GameState.prototype.isCeasefireActive = function()
+{
+	return this.ceasefireActive;
+};
+
 m.GameState.prototype.getTemplate = function(type)
 {
 	if (this.techTemplates[type] !== undefined)
 		return new m.Technology(this.techTemplates, type);
-	
+
 	if (!this.templates[type])
 		return null;
-	
-	return new m.Template(this.templates[type], this.techModifications);
+
+	return new m.Template(this.templates[type]);
 };
 
-m.GameState.prototype.applyCiv = function(str) {
+m.GameState.prototype.applyCiv = function(str)
+{
 	return str.replace(/\{civ\}/g, this.playerData.civ);
 };
 
-m.GameState.prototype.civ = function() {
-	return this.playerData.civ;
+m.GameState.prototype.getPlayerCiv = function(player)
+{
+	return player !== undefined ? this.sharedScript.playersData[player].civ : this.playerData.civ;
 };
 
 m.GameState.prototype.currentPhase = function()
@@ -165,9 +177,23 @@ m.GameState.prototype.cityPhase = function()
 	return this.phases[2].name;
 };
 
-m.GameState.prototype.getPhaseRequirements = function(i)
+m.GameState.prototype.getPhaseEntityRequirements = function(i)
 {
-	return this.phases[i-1].requirements ? this.phases[i-1].requirements : undefined;
+	let entityReqs = [];
+
+	for (let requirement of this.phases[i-1].requirements)
+	{
+		if (!requirement.entities)
+			continue;
+		for (let entity of requirement.entities)
+			if (entity.check == "count")
+				entityReqs.push({
+					"class": entity.class,
+					"count": entity.number
+				});
+	}
+
+	return entityReqs;
 };
 
 m.GameState.prototype.isResearched = function(template)
@@ -175,120 +201,109 @@ m.GameState.prototype.isResearched = function(template)
 	return this.playerData.researchedTechs[template] !== undefined;
 };
 
-// true if started or queued
+/** true if started or queued */
 m.GameState.prototype.isResearching = function(template)
 {
-	return (this.playerData.researchStarted[template] !== undefined || this.playerData.researchQueued[template] !== undefined);
+	return this.playerData.researchStarted[template] !== undefined ||
+	       this.playerData.researchQueued[template] !== undefined;
 };
 
-// this is an "in-absolute" check that doesn't check if we have a building to research from.
+/** this is an "in-absolute" check that doesn't check if we have a building to research from. */
 m.GameState.prototype.canResearch = function(techTemplateName, noRequirementCheck)
 {
-	var template = this.getTemplate(techTemplateName);
+	if (this.playerData.disabledTechnologies[techTemplateName])
+		return false;
+
+	let template = this.getTemplate(techTemplateName);
 	if (!template)
 		return false;
 
 	// researching or already researched: NOO.
-	if (this.playerData.researchQueued[techTemplateName] || this.playerData.researchStarted[techTemplateName] || this.playerData.researchedTechs[techTemplateName])
+	if (this.playerData.researchQueued[techTemplateName] ||
+	    this.playerData.researchStarted[techTemplateName] ||
+	    this.playerData.researchedTechs[techTemplateName])
 		return false;
 
 	if (noRequirementCheck === true)
 		return true;
-	
-	// not already researched, check if we can.
-	// basically a copy of the function in technologyManager since we can't use it.
-	// Checks the requirements for a technology to see if it can be researched at the current time
-		
-	// The technology which this technology supersedes is required
-	if (template.supersedes() && !this.playerData.researchedTechs[template.supersedes()])
-		return false;
 
-	// if this is a pair, we must check that the paire tech is not being researched
+	// if this is a pair, we must check that the pair tech is not being researched
 	if (template.pair())
 	{
-		var other = template.pairedWith();
-		if (this.playerData.researchQueued[other] || this.playerData.researchStarted[other] || this.playerData.researchedTechs[other])
+		let other = template.pairedWith();
+		if (this.playerData.researchQueued[other] ||
+		    this.playerData.researchStarted[other] ||
+		    this.playerData.researchedTechs[other])
 			return false;
 	}
 
-	return this.checkTechRequirements(template.requirements());
+	return this.checkTechRequirements(template.requirements(this.playerData.civ));
 };
 
-// Private function for checking a set of requirements is met
-// basically copies TechnologyManager's
-m.GameState.prototype.checkTechRequirements = function (reqs)
+/**
+ * Private function for checking a set of requirements is met.
+ * Basically copies TechnologyManager, but compares against
+ * variables only available within the AI
+ */
+m.GameState.prototype.checkTechRequirements = function(reqs)
 {
-	// If there are no requirements then all requirements are met
 	if (!reqs)
-		return true;
-	
-	if (reqs.tech)
-	{
-		return (this.playerData.researchedTechs[reqs.tech] !== undefined && this.playerData.researchedTechs[reqs.tech]);
-	}
-	else if (reqs.all)
-	{
-		for (var i = 0; i < reqs.all.length; i++)
-		{
-			if (!this.checkTechRequirements(reqs.all[i]))
-				return false;
-		}
-		return true;
-	}
-	else if (reqs.any)
-	{
-		for (var i = 0; i < reqs.any.length; i++)
-		{
-			if (this.checkTechRequirements(reqs.any[i]))
-				return true;
-		}
 		return false;
-	}
-	else if (reqs.class)
+
+	if (!reqs.length)
+		return true;
+
+	function doesEntitySpecPass(entity)
 	{
-		if (reqs.numberOfTypes)
+		switch (entity.check)
 		{
-			if (this.playerData.typeCountsByClass[reqs.class])
-				return (reqs.numberOfTypes <= Object.keys(this.playerData.typeCountsByClass[reqs.class]).length);
-			else
+		case "count":
+			if (!this.playerData.classCounts[entity.class] || this.playerData.classCounts[entity.class] < entity.number)
 				return false;
-		}
-		else if (reqs.number)
-		{
-			if (this.playerData.classCounts[reqs.class])
-				return (reqs.number <= this.playerData.classCounts[reqs.class]);
-			else
+			break;
+
+		case "variants":
+			if (!this.playerData.typeCountsByClass[entity.class] || Object.keys(this.playerData.typeCountsByClass[entity.class]).length < entity.number)
 				return false;
+			break;
 		}
+		return true;
 	}
-	else if (reqs.civ)
-	{
-		if (this.playerData.civ == reqs.civ)
-			return true;
-		else
+
+	return reqs.some(req => {
+		return Object.keys(req).every(type => {
+			switch (type)
+			{
+			case "techs":
+				return req[type].every(tech => !!this.playerData.researchedTechs[tech]);
+
+			case "entities":
+				return req[type].every(doesEntitySpecPass, this);
+			}
 			return false;
-	}
-	
-	// The technologies requirements are not a recognised format
-	error("Bad requirements " + uneval(reqs));
-	return false;
+		});
+	});
 };
 
-m.GameState.prototype.getMap = function() {
+m.GameState.prototype.getPassabilityMap = function()
+{
 	return this.sharedScript.passabilityMap;
 };
 
-m.GameState.prototype.getPassabilityClassMask = function(name) {
-	if (!(name in this.sharedScript.passabilityClasses))
+m.GameState.prototype.getPassabilityClassMask = function(name)
+{
+	if (!this.sharedScript.passabilityClasses[name])
 		error("Tried to use invalid passability class name '" + name + "'");
 	return this.sharedScript.passabilityClasses[name];
 };
 
-m.GameState.prototype.getResources = function() {
+m.GameState.prototype.getResources = function()
+{
 	return new m.Resources(this.playerData.resourceCounts);
 };
 
-m.GameState.prototype.getPopulation = function() {
+m.GameState.prototype.getPopulation = function()
+{
 	return this.playerData.popCount;
 };
 
@@ -300,22 +315,55 @@ m.GameState.prototype.getPopulationMax = function() {
 	return this.playerData.popMax;
 };
 
-m.GameState.prototype.getPlayerID = function() {
+m.GameState.prototype.getPlayerID = function()
+{
 	return this.player;
 };
 
-m.GameState.prototype.hasAllies = function() {
+m.GameState.prototype.hasAllies = function()
+{
 	for (let i in this.playerData.isAlly)
-		if (this.playerData.isAlly[i] && +i !== this.player)
+		if (this.playerData.isAlly[i] && +i !== this.player &&
+		    this.sharedScript.playersData[i].state !== "defeated")
 			return true;
 	return false;
 };
 
-m.GameState.prototype.isPlayerAlly = function(id) {
+m.GameState.prototype.hasEnemies = function()
+{
+	for (let i in this.playerData.isEnemy)
+		if (this.playerData.isEnemy[i] && +i !== 0 &&
+		    this.sharedScript.playersData[i].state !== "defeated")
+			return true;
+	return false;
+};
+
+m.GameState.prototype.hasNeutrals = function()
+{
+	for (let i in this.playerData.isNeutral)
+		if (this.playerData.isNeutral[i] &&
+		    this.sharedScript.playersData[i].state !== "defeated")
+			return true;
+	return false;
+};
+
+m.GameState.prototype.isPlayerNeutral = function(id)
+{
+	return this.playerData.isNeutral[id];
+};
+
+m.GameState.prototype.isPlayerAlly = function(id)
+{
 	return this.playerData.isAlly[id];
 };
 
-m.GameState.prototype.isPlayerEnemy = function(id) {
+m.GameState.prototype.isPlayerMutualAlly = function(id)
+{
+	return this.playerData.isMutualAlly[id];
+};
+
+m.GameState.prototype.isPlayerEnemy = function(id)
+{
 	return this.playerData.isEnemy[id];
 };
 
@@ -328,6 +376,15 @@ m.GameState.prototype.getEnemies = function()
 	return ret;
 };
 
+m.GameState.prototype.getNeutrals = function()
+{
+	let ret = [];
+	for (let i in this.playerData.isNeutral)
+		if (this.playerData.isNeutral[i])
+			ret.push(+i);
+	return ret;
+};
+
 m.GameState.prototype.getAllies = function()
 {
 	let ret = [];
@@ -346,44 +403,42 @@ m.GameState.prototype.getExclusiveAllies = function()
 	return ret;
 };
 
+m.GameState.prototype.getMutualAllies = function()
+{
+	let ret = [];
+	for (let i in this.playerData.isMutualAlly)
+		if (this.playerData.isMutualAlly[i] &&
+		    this.sharedScript.playersData[i].isMutualAlly[this.player])
+			ret.push(+i);
+	return ret;
+};
+
 m.GameState.prototype.isEntityAlly = function(ent)
 {
-	if (!ent || !ent.owner)
+	if (!ent)
 		return false;
-	if ((typeof ent.owner) === "function")
-		return this.playerData.isAlly[ent.owner()];
-	else
-		return this.playerData.isAlly[ent.owner];
+	return this.playerData.isAlly[ent.owner()];
 };
 
 m.GameState.prototype.isEntityExclusiveAlly = function(ent)
 {
-	if (!ent || !ent.owner)
+	if (!ent)
 		return false;
-	if ((typeof ent.owner) === "function")
-		return (this.playerData.isAlly[ent.owner()] && ent.owner() !== this.player);
-	else
-		return (this.playerData.isAlly[ent.owner] && ent.owner !== this.player);
+	return this.playerData.isAlly[ent.owner()] && ent.owner() !== this.player;
 };
 
 m.GameState.prototype.isEntityEnemy = function(ent)
 {
-	if (!ent || !ent.owner)
+	if (!ent)
 		return false;
-	if ((typeof ent.owner) === "function")
-		return this.playerData.isEnemy[ent.owner()];
-	else
-		return this.playerData.isEnemy[ent.owner];
+	return this.playerData.isEnemy[ent.owner()];
 };
- 
+
 m.GameState.prototype.isEntityOwn = function(ent)
 {
-	if (!ent || !ent.owner)
+	if (!ent)
 		return false;
-	if ((typeof ent.owner) === "function")
-		return ent.owner() === this.player;
-	else
-		return ent.owner === this.player;
+	return ent.owner() === this.player;
 };
 
 m.GameState.prototype.getEntityById = function(id)
@@ -394,70 +449,101 @@ m.GameState.prototype.getEntityById = function(id)
 	return undefined;
 };
 
-m.GameState.prototype.getEntities = function() {
-	return this.entities;
+m.GameState.prototype.getEntities = function(id)
+{
+	if (id === undefined)
+		return this.entities;
+
+	return this.updatingGlobalCollection("" + id + "-entities", m.Filters.byOwner(id));
 };
 
-m.GameState.prototype.getOwnEntities = function() {
+m.GameState.prototype.getStructures = function()
+{
+	return this.updatingGlobalCollection("structures", m.Filters.byClass("Structure"), this.entities);
+};
+
+m.GameState.prototype.getOwnEntities = function()
+{
 	return this.updatingGlobalCollection("" + this.player + "-entities", m.Filters.byOwner(this.player));
 };
 
-m.GameState.prototype.getOwnStructures = function() {
+m.GameState.prototype.getOwnStructures = function()
+{
 	return this.updatingGlobalCollection("" + this.player + "-structures", m.Filters.byClass("Structure"), this.getOwnEntities());
 };
 
-m.GameState.prototype.getOwnUnits = function() {
+m.GameState.prototype.getOwnUnits = function()
+{
 	return this.updatingGlobalCollection("" + this.player + "-units", m.Filters.byClass("Unit"), this.getOwnEntities());
 };
 
-m.GameState.prototype.getAllyEntities = function() {
+m.GameState.prototype.getAllyEntities = function()
+{
 	return this.entities.filter(m.Filters.byOwners(this.getAllies()));
 };
 
-m.GameState.prototype.getExclusiveAllyEntities = function() {
+m.GameState.prototype.getExclusiveAllyEntities = function()
+{
 	return this.entities.filter(m.Filters.byOwners(this.getExclusiveAllies()));
 };
 
-m.GameState.prototype.getAllyStructures = function() {
+m.GameState.prototype.getAllyStructures = function()
+{
 	return this.updatingCollection("ally-structures", m.Filters.byClass("Structure"), this.getAllyEntities());
 };
 
-// Try to use a parameter for those three, it'll be a lot faster.
-
-m.GameState.prototype.getEnemyEntities = function(enemyID) {
-	if (enemyID === undefined)
-		return this.entities.filter(m.Filters.byOwners(this.getEnemies()));
-
-	return this.updatingGlobalCollection("" + enemyID + "-entities", m.Filters.byOwner(enemyID));
+m.GameState.prototype.resetAllyStructures = function()
+{
+	this.destroyCollection("ally-structures");
 };
 
-m.GameState.prototype.getEnemyStructures = function(enemyID) {
+m.GameState.prototype.getNeutralStructures = function()
+{
+	return this.getStructures().filter(m.Filters.byOwners(this.getNeutrals()));
+};
+
+m.GameState.prototype.getEnemyEntities = function()
+{
+	return this.entities.filter(m.Filters.byOwners(this.getEnemies()));
+};
+
+m.GameState.prototype.getEnemyStructures = function(enemyID)
+{
 	if (enemyID === undefined)
 		return this.updatingCollection("enemy-structures", m.Filters.byClass("Structure"), this.getEnemyEntities());
 
-	return this.updatingGlobalCollection("" + enemyID + "-structures", m.Filters.byClass("Structure"), this.getEnemyEntities(enemyID));
+	return this.updatingGlobalCollection("" + enemyID + "-structures", m.Filters.byClass("Structure"), this.getEntities(enemyID));
 };
 
-m.GameState.prototype.getEnemyUnits = function(enemyID) {
+m.GameState.prototype.resetEnemyStructures = function()
+{
+	this.destroyCollection("enemy-structures");
+};
+
+m.GameState.prototype.getEnemyUnits = function(enemyID)
+{
 	if (enemyID === undefined)
 		return this.getEnemyEntities().filter(m.Filters.byClass("Unit"));
 
-	return this.updatingGlobalCollection("" + enemyID + "-units", m.Filters.byClass("Unit"), this.getEnemyEntities(enemyID));
+	return this.updatingGlobalCollection("" + enemyID + "-units", m.Filters.byClass("Unit"), this.getEntities(enemyID));
 };
 
-// if maintain is true, this will be stored. Otherwise it's one-shot.
-m.GameState.prototype.getOwnEntitiesByMetadata = function(key, value, maintain){
+/** if maintain is true, this will be stored. Otherwise it's one-shot. */
+m.GameState.prototype.getOwnEntitiesByMetadata = function(key, value, maintain)
+{
 	if (maintain === true)
 		return this.updatingCollection(key + "-" + value, m.Filters.byMetadata(this.player, key, value),this.getOwnEntities());
 	return this.getOwnEntities().filter(m.Filters.byMetadata(this.player, key, value));
 };
 
-m.GameState.prototype.getOwnEntitiesByRole = function(role, maintain){
+m.GameState.prototype.getOwnEntitiesByRole = function(role, maintain)
+{
 	return this.getOwnEntitiesByMetadata("role", role, maintain);
 };
 
-m.GameState.prototype.getOwnEntitiesByType = function(type, maintain){
-	var filter = m.Filters.byType(type);
+m.GameState.prototype.getOwnEntitiesByType = function(type, maintain)
+{
+	let filter = m.Filters.byType(type);
 	if (maintain === true)
 		return this.updatingCollection("type-" + type, filter, this.getOwnEntities());
 	return this.getOwnEntities().filter(filter);
@@ -465,7 +551,7 @@ m.GameState.prototype.getOwnEntitiesByType = function(type, maintain){
 
 m.GameState.prototype.getOwnEntitiesByClass = function(cls, maintain)
 {
-	var filter = m.Filters.byClass(cls);
+	let filter = m.Filters.byClass(cls);
 	if (maintain)
 		return this.updatingCollection("class-" + cls, filter, this.getOwnEntities());
 	return this.getOwnEntities().filter(filter);
@@ -473,32 +559,36 @@ m.GameState.prototype.getOwnEntitiesByClass = function(cls, maintain)
 
 m.GameState.prototype.getOwnFoundationsByClass = function(cls, maintain)
 {
-	var filter = m.Filters.byClass(cls);
+	let filter = m.Filters.byClass(cls);
 	if (maintain)
 		return this.updatingCollection("foundations-class-" + cls, filter, this.getOwnFoundations());
 	return this.getOwnFoundations().filter(filter);
 };
 
-m.GameState.prototype.getOwnTrainingFacilities = function(){
+m.GameState.prototype.getOwnTrainingFacilities = function()
+{
 	return this.updatingGlobalCollection("" + this.player + "-training-facilities", m.Filters.byTrainingQueue(), this.getOwnEntities());
 };
 
-m.GameState.prototype.getOwnResearchFacilities = function(){
+m.GameState.prototype.getOwnResearchFacilities = function()
+{
 	return this.updatingGlobalCollection("" + this.player + "-research-facilities", m.Filters.byResearchAvailable(this.playerData.civ), this.getOwnEntities());
 };
 
 
-m.GameState.prototype.countEntitiesByType = function(type, maintain) {
+m.GameState.prototype.countEntitiesByType = function(type, maintain)
+{
 	return this.getOwnEntitiesByType(type, maintain).length;
 };
 
-m.GameState.prototype.countEntitiesAndQueuedByType = function(type, maintain) {
-	var template = this.getTemplate(type);
+m.GameState.prototype.countEntitiesAndQueuedByType = function(type, maintain)
+{
+	let template = this.getTemplate(type);
 	if (!template)
 		return 0;
 
-	var count = this.countEntitiesByType(type, maintain);
-	
+	let count = this.countEntitiesByType(type, maintain);
+
 	// Count building foundations
 	if (template.hasClass("Structure") === true)
 		count += this.countFoundationsByType(type, true);
@@ -509,23 +599,23 @@ m.GameState.prototype.countEntitiesAndQueuedByType = function(type, maintain) {
 		// Count entities in building production queues
 		// TODO: maybe this fails for corrals.
 		this.getOwnTrainingFacilities().forEach(function(ent) {
-			ent.trainingQueue().forEach(function(item) {
+			for (let item of ent.trainingQueue())
 				if (item.unitTemplate == type)
 					count += item.count;
-			});
 		});
 	}
-	
+
 	return count;
 };
 
-m.GameState.prototype.countFoundationsByType = function(type, maintain) {
-	var foundationType = "foundation|" + type;
+m.GameState.prototype.countFoundationsByType = function(type, maintain)
+{
+	let foundationType = "foundation|" + type;
 
 	if (maintain === true)
 		return this.updatingCollection("foundation-type-" + type, m.Filters.byType(foundationType), this.getOwnFoundations()).length;
 
-	var count = 0;
+	let count = 0;
 	this.getOwnStructures().forEach(function(ent) {
 		if (ent.templateName() == foundationType)
 			++count;
@@ -533,82 +623,94 @@ m.GameState.prototype.countFoundationsByType = function(type, maintain) {
 	return count;
 };
 
-m.GameState.prototype.countOwnEntitiesByRole = function(role) {
-	return this.getOwnEntitiesByRole(role).length;
+m.GameState.prototype.countOwnEntitiesByRole = function(role)
+{
+	return this.getOwnEntitiesByRole(role, "true").length;
 };
 
-m.GameState.prototype.countOwnEntitiesAndQueuedWithRole = function(role) {
-	var count = this.countOwnEntitiesByRole(role);
-	
+m.GameState.prototype.countOwnEntitiesAndQueuedWithRole = function(role)
+{
+	let count = this.countOwnEntitiesByRole(role);
+
 	// Count entities in building production queues
 	this.getOwnTrainingFacilities().forEach(function(ent) {
-		ent.trainingQueue().forEach(function(item) {
-			if (item.metadata && item.metadata.role == role)
+		for (let item of ent.trainingQueue())
+			if (item.metadata && item.metadata.role && item.metadata.role == role)
 				count += item.count;
-		});
 	});
 	return count;
 };
 
-m.GameState.prototype.countOwnQueuedEntitiesWithMetadata = function(data, value) {
+m.GameState.prototype.countOwnQueuedEntitiesWithMetadata = function(data, value)
+{
 	// Count entities in building production queues
-	var count = 0;
+	let count = 0;
 	this.getOwnTrainingFacilities().forEach(function(ent) {
-		ent.trainingQueue().forEach(function(item) {
+		for (let item of ent.trainingQueue())
 			if (item.metadata && item.metadata[data] && item.metadata[data] == value)
 				count += item.count;
-		});
 	});
 	return count;
 };
 
-m.GameState.prototype.getOwnFoundations = function() {
+m.GameState.prototype.getOwnFoundations = function()
+{
 	return this.updatingGlobalCollection("" + this.player + "-foundations", m.Filters.isFoundation(), this.getOwnStructures());
 };
 
-m.GameState.prototype.getOwnDropsites = function(resource){
+m.GameState.prototype.getOwnDropsites = function(resource)
+{
 	if (resource !== undefined)
-		return this.updatingCollection("dropsite-" + resource, m.Filters.isDropsite(resource), this.getOwnEntities());
-	return this.updatingCollection("dropsite-all", m.Filters.isDropsite(), this.getOwnEntities());
+		return this.updatingCollection("ownDropsite-" + resource, m.Filters.isDropsite(resource), this.getOwnEntities());
+	return this.updatingCollection("ownDropsite-all", m.Filters.isDropsite(), this.getOwnEntities());
 };
 
-m.GameState.prototype.getResourceSupplies = function(resource){
+m.GameState.prototype.getAnyDropsites = function(resource)
+{
+	if (resource !== undefined)
+		return this.updatingGlobalCollection("anyDropsite-" + resource, m.Filters.isDropsite(resource), this.getEntities());
+	return this.updatingGlobalCollection("anyDropsite-all", m.Filters.isDropsite(), this.getEntities());
+};
+
+m.GameState.prototype.getResourceSupplies = function(resource)
+{
 	return this.updatingGlobalCollection("resource-" + resource, m.Filters.byResource(resource), this.getEntities());
 };
 
-m.GameState.prototype.getHuntableSupplies = function(){
+m.GameState.prototype.getHuntableSupplies = function()
+{
 	return this.updatingGlobalCollection("resource-hunt", m.Filters.isHuntable(), this.getEntities());
 };
 
-m.GameState.prototype.getFishableSupplies = function(){
+m.GameState.prototype.getFishableSupplies = function()
+{
 	return this.updatingGlobalCollection("resource-fish", m.Filters.isFishable(), this.getEntities());
 };
 
-// This returns only units from buildings.
+/** This returns only units from buildings. */
 m.GameState.prototype.findTrainableUnits = function(classes, anticlasses)
 {
-	var allTrainable = [];
-	var civ = this.playerData.civ;
+	let allTrainable = [];
+	let civ = this.playerData.civ;
 	this.getOwnStructures().forEach(function(ent) {
-		var trainable = ent.trainableEntities(civ);
+		let trainable = ent.trainableEntities(civ);
 		if (!trainable)
 			return;
 		for (let unit of trainable)
 			if (allTrainable.indexOf(unit) === -1)
 				allTrainable.push(unit);
 	});
-	var ret = [];
-	var limits = this.getEntityLimits();
-	var current = this.getEntityCounts();
+	let ret = [];
+	let limits = this.getEntityLimits();
+	let current = this.getEntityCounts();
 	for (let trainable of allTrainable)
 	{
+		if (this.isTemplateDisabled(trainable))
+			continue;
 		let template = this.getTemplate(trainable);
-
 		if (!template || !template.available(this))
 			continue;
-		if (this.isDisabledTemplates(trainable))
-			continue;
-		
+
 		let okay = true;
 		for (let clas of classes)
 		{
@@ -639,22 +741,25 @@ m.GameState.prototype.findTrainableUnits = function(classes, anticlasses)
 	return ret;
 };
 
-// Return all techs which can currently be researched
-// Does not factor cost.
-// If there are pairs, both techs are returned.
+/**
+ * Return all techs which can currently be researched
+ * Does not factor cost.
+ * If there are pairs, both techs are returned.
+ */
 m.GameState.prototype.findAvailableTech = function()
-{	
-	var allResearchable = [];
-	var civ = this.playerData.civ;
-	this.getOwnEntities().forEach(function(ent) {
+{
+	let allResearchable = [];
+	let civ = this.playerData.civ;
+	for (let ent of this.getOwnEntities().values())
+	{
 		let searchable = ent.researchableTechs(civ);
 		if (!searchable)
-			return;
+			continue;
 		for (let tech of searchable)
-			if (allResearchable.indexOf(tech) === -1)
+			if (!this.playerData.disabledTechnologies[tech] && allResearchable.indexOf(tech) === -1)
 				allResearchable.push(tech);
-	});
-	
+	}
+
 	let ret = [];
 	for (let tech of allResearchable)
 	{
@@ -668,64 +773,107 @@ m.GameState.prototype.findAvailableTech = function()
 				ret.push([techs[1]._templateName, techs[1]] );
 		}
 		else
-			if (this.canResearch(tech) && template._templateName != this.townPhase() && template._templateName != this.cityPhase())
+			if (this.canResearch(tech) && template._templateName != this.townPhase() &&
+			                              template._templateName != this.cityPhase())
 				ret.push( [tech, template] );
 	}
 	return ret;
 };
 
 /**
- * Find buildings that are capable of training that template.
+ * Return true if we have a building able to train that template
+ */
+m.GameState.prototype.hasTrainer = function(template)
+{
+	let civ = this.playerData.civ;
+	for (let ent of this.getOwnTrainingFacilities().values())
+	{
+		let trainable = ent.trainableEntities(civ);
+		if (trainable && trainable.indexOf(template) !== -1)
+			return true;
+	}
+	return false;
+};
+
+/**
+ * Find buildings able to train that template.
  */
 m.GameState.prototype.findTrainers = function(template)
-{	
-	var civ = this.playerData.civ;
+{
+	let civ = this.playerData.civ;
 	return this.getOwnTrainingFacilities().filter(function(ent) {
 		let trainable = ent.trainableEntities(civ);
-		if (!trainable || trainable.indexOf(template) == -1)
-			return false;
-		return true;
+		return trainable && trainable.indexOf(template) !== -1;
 	});
 };
 
 /**
- * Find units that are capable of constructing the given building type.
+ * Get any unit that is capable of constructing the given building type.
  */
-m.GameState.prototype.findBuilders = function(template)
+m.GameState.prototype.findBuilder = function(template)
 {
-	return this.getOwnUnits().filter(function(ent) {
+	for (let ent of this.getOwnUnits().values())
+	{
 		let buildable = ent.buildableEntities();
-		if (!buildable || buildable.indexOf(template) == -1)
-			return false;
-		return true;
-	});
+		if (buildable && buildable.indexOf(template) !== -1)
+			return ent;
+	}
+	return undefined;
 };
 
-// Find buildings that are capable of researching the given tech
+/** Return true if one of our buildings is capable of researching the given tech */
+m.GameState.prototype.hasResearchers = function(templateName, noRequirementCheck)
+{
+	// let's check we can research the tech.
+	if (!this.canResearch(templateName, noRequirementCheck))
+		return false;
+
+	let civ = this.playerData.civ;
+
+	for (let ent of this.getOwnResearchFacilities().values())
+	{
+		let techs = ent.researchableTechs(civ);
+		for (let tech of techs)
+		{
+			let temp = this.getTemplate(tech);
+			if (temp.pairDef())
+			{
+				let pairedTechs = temp.getPairedTechs();
+				if (pairedTechs[0]._templateName == templateName ||
+				    pairedTechs[1]._templateName == templateName)
+					return true;
+			}
+			else if (tech == templateName)
+				return true;
+		}
+	}
+	return false;
+};
+
+/** Find buildings that are capable of researching the given tech */
 m.GameState.prototype.findResearchers = function(templateName, noRequirementCheck)
 {
 	// let's check we can research the tech.
 	if (!this.canResearch(templateName, noRequirementCheck))
-		return [];
+		return undefined;
+
+	let self = this;
+	let civ = this.playerData.civ;
 
-	var template = this.getTemplate(templateName);
-	var self = this;
-	var civ = this.playerData.civ;
-	
 	return this.getOwnResearchFacilities().filter(function(ent) {
-		var techs = ent.researchableTechs(civ);
+		let techs = ent.researchableTechs(civ);
 		for (let tech of techs)
 		{
 			let thisTemp = self.getTemplate(tech);
 			if (thisTemp.pairDef())
 			{
-				var pairedTechs = thisTemp.getPairedTechs();
-				if (pairedTechs[0]._templateName == templateName || pairedTechs[1]._templateName == templateName)
+				let pairedTechs = thisTemp.getPairedTechs();
+				if (pairedTechs[0]._templateName == templateName ||
+				    pairedTechs[1]._templateName == templateName)
 					return true;
 			}
-			else
-				if (tech == templateName)
-					return true;
+			else if (tech == templateName)
+				return true;
 		}
 		return false;
 	});
@@ -741,19 +889,32 @@ m.GameState.prototype.getEntityCounts = function()
 	return this.playerData.entityCounts;
 };
 
-m.GameState.prototype.isDisabledTemplates = function(template)
+m.GameState.prototype.isTemplateDisabled = function(template)
 {
 	if (!this.playerData.disabledTemplates[template])
 		return false;
 	return this.playerData.disabledTemplates[template];
 };
 
-// Checks whether the maximum number of buildings have been cnstructed for a certain catergory
+/** Checks whether the maximum number of buildings have been constructed for a certain catergory */
 m.GameState.prototype.isEntityLimitReached = function(category)
 {
-	if(this.playerData.entityLimits[category] === undefined || this.playerData.entityCounts[category] === undefined)
+	if (this.playerData.entityLimits[category] === undefined ||
+	    this.playerData.entityCounts[category] === undefined)
 		return false;
-	return (this.playerData.entityCounts[category] >= this.playerData.entityLimits[category]);
+	return this.playerData.entityCounts[category] >= this.playerData.entityLimits[category];
+};
+
+m.GameState.prototype.getTraderTemplatesGains = function()
+{
+	let shipMechantTemplateName = this.applyCiv("units/{civ}_ship_merchant");
+	let supportTraderTemplateName = this.applyCiv("units/{civ}_support_trader");
+	let shipMerchantTemplate = !this.isTemplateDisabled(shipMechantTemplateName) && this.getTemplate(shipMechantTemplateName);
+	let supportTraderTemplate = !this.isTemplateDisabled(supportTraderTemplateName) && this.getTemplate(supportTraderTemplateName);
+	return {
+		"navalGainMultiplier": shipMerchantTemplate && shipMerchantTemplate.gainMultiplier(),
+		"landGainMultiplier": supportTraderTemplate && supportTraderTemplate.gainMultiplier()
+	};
 };
 
 return m;
