@@ -6,10 +6,11 @@ m.Resources = function(amounts = {}, population = 0)
 	for (let key of this.types)
 		this[key] = amounts[key] || 0;
 
-	this.population = (population > 0) ? population : 0;
+	this.population = population > 0 ? population : 0;
 };
 
-m.Resources.prototype.types = [ "food", "wood", "stone", "metal" ];
+// This array will be filled in SharedScript.init
+m.Resources.prototype.types = [];
 
 m.Resources.prototype.reset = function()
 {
@@ -47,15 +48,6 @@ m.Resources.prototype.multiply = function(n)
 	this.population *= n;
 };
 
-m.Resources.prototype.toInt = function()
-{
-	let sum = 0;
-	for (let key of this.types)
-		sum += this[key];
-	sum += this.population * 50; // based on typical unit costs
-	return sum;
-};
-
 m.Resources.prototype.Serialize = function()
 {
 	let amounts = {};
