@@ -1,8 +1,7 @@
 var PETRA = function(m)
 {
 
-/* Attack Manager
- */
+/** Attack Manager */
 
 m.AttackManager = function(Config)
 {
@@ -21,7 +20,7 @@ m.AttackManager = function(Config)
 	this.defeated = {};
 };
 
-// More initialisation for stuff that needs the gameState
+/** More initialisation for stuff that needs the gameState */
 m.AttackManager.prototype.init = function(gameState)
 {
 	this.outOfPlan = gameState.getOwnUnits().filter(API3.Filters.byMetadata(PlayerID, "plan", -1));
@@ -49,17 +48,17 @@ m.AttackManager.prototype.setRushes = function(allowed)
 
 m.AttackManager.prototype.checkEvents = function(gameState, events)
 {
-	for (let evt of events["PlayerDefeated"])
+	for (let evt of events.PlayerDefeated)
 		this.defeated[evt.playerId] = true;
 
-	let answer = false;
+	let answer = "decline";
 	let other;
 	let targetPlayer;
-	for (let evt of events["AttackRequest"])
+	for (let evt of events.AttackRequest)
 	{
-		if (evt.source === PlayerID || !gameState.isPlayerAlly(evt.source) || !gameState.isPlayerEnemy(evt.target))
+		if (evt.source === PlayerID || !gameState.isPlayerAlly(evt.source) || !gameState.isPlayerEnemy(evt.player))
 			continue;
-		targetPlayer = evt.target;
+		targetPlayer = evt.player;
 		let available = 0;
 		for (let attackType in this.upcomingAttacks)
 		{
@@ -87,24 +86,28 @@ m.AttackManager.prototype.checkEvents = function(gameState, events)
 			{
 				for (let attack of this.upcomingAttacks[attackType])
 				{
-					if (attack.state === "completing"
-						|| attack.targetPlayer !== targetPlayer
-						|| attack.unitCollection.length < 3)
+					if (attack.state === "completing" ||
+						attack.targetPlayer !== targetPlayer ||
+						attack.unitCollection.length < 3)
 						continue;
 					attack.forceStart();
 					attack.requested = true;
 				}
 			}
-			answer = true;
+			answer = "join";
 		}
+		else if (other !== undefined)
+			answer = "other";
 		break;  // take only the first attack request into account
 	}
 	if (targetPlayer !== undefined)
 		m.chatAnswerRequestAttack(gameState, targetPlayer, answer, other);
 };
 
-// Some functions are run every turn
-// Others once in a while
+/**
+ * Some functions are run every turn
+ * Others once in a while
+ */
 m.AttackManager.prototype.update = function(gameState, queues, events)
 {
 	if (this.Config.debug > 2 && gameState.ai.elapsedTime > this.debugTime + 60)
@@ -113,7 +116,7 @@ m.AttackManager.prototype.update = function(gameState, queues, events)
 		API3.warn(" upcoming attacks =================");
 		for (let attackType in this.upcomingAttacks)
 			for (let attack of this.upcomingAttacks[attackType])
-				API3.warn(" plan " + attack.name + " type " + attackType + " state " + attack.state  + " units " + attack.unitCollection.length);
+				API3.warn(" plan " + attack.name + " type " + attackType + " state " + attack.state + " units " + attack.unitCollection.length);
 		API3.warn(" started attacks ==================");
 		for (let attackType in this.startedAttacks)
 			for (let attack of this.startedAttacks[attackType])
@@ -156,7 +159,7 @@ m.AttackManager.prototype.update = function(gameState, queues, events)
 					if (this.Config.debug > 1)
 						API3.warn("Attack Manager: Starting " + attack.getType() + " plan " + attack.getName());
 					if (this.Config.chat)
-						m.chatLaunchAttack(gameState, attack.targetPlayer);
+						m.chatLaunchAttack(gameState, attack.targetPlayer, attack.getType());
 					this.startedAttacks[attackType].push(attack);
 				}
 				else
@@ -188,37 +191,33 @@ m.AttackManager.prototype.update = function(gameState, queues, events)
 
 	// creating plans after updating because an aborted plan might be reused in that case.
 
-	var barracksNb = gameState.getOwnEntitiesByClass("Barracks", true).filter(API3.Filters.isBuilt()).length;
+	let barracksNb = gameState.getOwnEntitiesByClass("Barracks", true).filter(API3.Filters.isBuilt()).length;
 	if (this.rushNumber < this.maxRushes && barracksNb >= 1)
 	{
-		if (unexecutedAttacks["Rush"] === 0)
+		if (unexecutedAttacks.Rush === 0)
 		{
 			// we have a barracks and we want to rush, rush.
-			var data = { "targetSize": this.rushSize[this.rushNumber] };
-			var attackPlan = new m.AttackPlan(gameState, this.Config, this.totalNumber, "Rush", data);
+			let data = { "targetSize": this.rushSize[this.rushNumber] };
+			let attackPlan = new m.AttackPlan(gameState, this.Config, this.totalNumber, "Rush", data);
 			if (!attackPlan.failed)
 			{
 				if (this.Config.debug > 1)
 					API3.warn("Headquarters: Rushing plan " + this.totalNumber + " with maxRushes " + this.maxRushes);
 				this.totalNumber++;
 				attackPlan.init(gameState);
-				this.upcomingAttacks["Rush"].push(attackPlan);
+				this.upcomingAttacks.Rush.push(attackPlan);
 			}
 			this.rushNumber++;
 		}
 	}
-	else if (unexecutedAttacks["Attack"] === 0 && unexecutedAttacks["HugeAttack"] === 0
-		&& (this.startedAttacks["Attack"].length + this.startedAttacks["HugeAttack"].length < Math.min(2, 1 + Math.round(gameState.getPopulationMax()/100))))
+	else if (unexecutedAttacks.Attack === 0 && unexecutedAttacks.HugeAttack === 0 &&
+		this.startedAttacks.Attack.length + this.startedAttacks.HugeAttack.length < Math.min(2, 1 + Math.round(gameState.getPopulationMax()/100)))
 	{
-		if ((barracksNb >= 1 && (gameState.currentPhase() > 1 || gameState.isResearching(gameState.townPhase())))
-			|| !gameState.ai.HQ.baseManagers[1])	// if we have no base ... nothing else to do than attack
+		if ((barracksNb >= 1 && (gameState.currentPhase() > 1 || gameState.isResearching(gameState.townPhase()))) ||
+			!gameState.ai.HQ.baseManagers[1])	// if we have no base ... nothing else to do than attack
 		{
-			if (this.attackNumber < 2 || this.startedAttacks["HugeAttack"].length > 0)
-				var type = "Attack";
-			else
-				var type = "HugeAttack";
-
-			var attackPlan = new m.AttackPlan(gameState, this.Config, this.totalNumber, type);
+			let type = this.attackNumber < 2 || this.startedAttacks.HugeAttack.length > 0 ? "Attack" : "HugeAttack";
+			let attackPlan = new m.AttackPlan(gameState, this.Config, this.totalNumber, type);
 			if (attackPlan.failed)
 				this.attackPlansEncounteredWater = true; // hack
 			else
@@ -233,44 +232,34 @@ m.AttackManager.prototype.update = function(gameState, queues, events)
 		}
 	}
 
-	if (unexecutedAttacks["Raid"] === 0 && gameState.ai.HQ.defenseManager.targetList.length)
+	if (unexecutedAttacks.Raid === 0 && gameState.ai.HQ.defenseManager.targetList.length)
 	{
-		var target = undefined;
-		for (var targetId of gameState.ai.HQ.defenseManager.targetList)
+		let target;
+		for (let targetId of gameState.ai.HQ.defenseManager.targetList)
 		{
 			target = gameState.getEntityById(targetId);
-			if (target)
+			if (!target)
+				continue;
+			if (gameState.isPlayerEnemy(target.owner()))
 				break;
+			target = undefined;
 		}
-		if (target)
-		{
-			// prepare a raid against this target
-			var data = { "target": target };
-			var attackPlan = new m.AttackPlan(gameState, this.Config, this.totalNumber, "Raid", data);
-			if (!attackPlan.failed)
-			{
-				if (this.Config.debug > 1)
-					API3.warn("Headquarters: Raiding plan " + this.totalNumber);
-				this.totalNumber++;
-				attackPlan.init(gameState);
-				this.upcomingAttacks["Raid"].push(attackPlan);
-			}
-			this.raidNumber++;
-		}
+		if (target) // prepare a raid against this target
+			this.raidTargetEntity(gameState, target);
 	}
 };
 
 m.AttackManager.prototype.getPlan = function(planName)
 {
-	for (var attackType in this.upcomingAttacks)
+	for (let attackType in this.upcomingAttacks)
 	{
-		for (var attack of this.upcomingAttacks[attackType])
+		for (let attack of this.upcomingAttacks[attackType])
 			if (attack.getName() == planName)
 				return attack;
 	}
-	for (var attackType in this.startedAttacks)
+	for (let attackType in this.startedAttacks)
 	{
-		for (var attack of this.startedAttacks[attackType])
+		for (let attack of this.startedAttacks[attackType])
 			if (attack.getName() == planName)
 				return attack;
 	}
@@ -279,37 +268,37 @@ m.AttackManager.prototype.getPlan = function(planName)
 
 m.AttackManager.prototype.pausePlan = function(planName)
 {
-	var attack = this.getPlan(planName);
+	let attack = this.getPlan(planName);
 	if (attack)
 		attack.setPaused(true);
 };
 
 m.AttackManager.prototype.unpausePlan = function(planName)
 {
-	var attack = this.getPlan(planName);
+	let attack = this.getPlan(planName);
 	if (attack)
 		attack.setPaused(false);
 };
 
 m.AttackManager.prototype.pauseAllPlans = function()
 {
-	for (var attackType in this.upcomingAttacks)
-		for (var attack of this.upcomingAttacks[attackType])
+	for (let attackType in this.upcomingAttacks)
+		for (let attack of this.upcomingAttacks[attackType])
 			attack.setPaused(true);
 
-	for (var attackType in this.startedAttacks)
-		for (var attack of this.startedAttacks[attackType])
+	for (let attackType in this.startedAttacks)
+		for (let attack of this.startedAttacks[attackType])
 			attack.setPaused(true);
 };
 
 m.AttackManager.prototype.unpauseAllPlans = function()
 {
-	for (var attackType in this.upcomingAttacks)
-		for (var attack of this.upcomingAttacks[attackType])
+	for (let attackType in this.upcomingAttacks)
+		for (let attack of this.upcomingAttacks[attackType])
 			attack.setPaused(false);
 
-	for (var attackType in this.startedAttacks)
-		for (var attack of this.startedAttacks[attackType])
+	for (let attackType in this.startedAttacks)
+		for (let attack of this.startedAttacks[attackType])
 			attack.setPaused(false);
 };
 
@@ -327,14 +316,14 @@ m.AttackManager.prototype.getAttackInPreparation = function(type)
  */
 m.AttackManager.prototype.getEnemyPlayer = function(gameState, attack)
 {
-	var enemyPlayer;
+	let enemyPlayer;
 
 	// first check if there is a preferred enemy based on our victory conditions
 	if (gameState.getGameType() === "wonder")
 	{
-		var moreAdvanced;
-		var enemyWonder;
-		var wonders = gameState.getEnemyStructures().filter(API3.Filters.byClass("Wonder"));
+		let moreAdvanced;
+		let enemyWonder;
+		let wonders = gameState.getEnemyStructures().filter(API3.Filters.byClass("Wonder"));
 		for (let wonder of wonders.values())
 		{
 			if (wonder.owner() === 0)
@@ -358,11 +347,37 @@ m.AttackManager.prototype.getEnemyPlayer = function(gameState, attack)
 			return enemyPlayer;
 		}
 	}
+	else if (gameState.getGameType() === "capture_the_relic")
+	{
+		// Target the player with the most relics (including gaia)
+		let allRelics = gameState.updatingGlobalCollection("allRelics", API3.Filters.byClass("Relic"));
+		let maxRelicsOwned = 0;
+		for (let i = 0; i < gameState.sharedScript.playersData.length; ++i)
+		{
+			if (!gameState.isPlayerEnemy(i) || this.defeated[i] ||
+			    i === 0 && !gameState.ai.HQ.gameTypeManager.tryCaptureGaiaRelic)
+				continue;
 
-	var veto = {};
+			let relicsCount = allRelics.filter(relic => relic.owner() === i).length;
+			if (relicsCount <= maxRelicsOwned)
+				continue;
+			maxRelicsOwned = relicsCount;
+			enemyPlayer = i;
+		}
+		if (enemyPlayer !== undefined)
+		{
+			if (attack.targetPlayer === undefined)
+				this.currentEnemyPlayer = enemyPlayer;
+			if (enemyPlayer === 0)
+				gameState.ai.HQ.gameTypeManager.resetCaptureGaiaRelic(gameState);
+			return enemyPlayer;
+		}
+	}
+
+	let veto = {};
 	for (let i in this.defeated)
 		veto[i] = true;
-	// No rush if enemy too well defended (i.e. iberians)     
+	// No rush if enemy too well defended (i.e. iberians)
 	if (attack.type === "Rush")
 	{
 		for (let i = 1; i < gameState.sharedScript.playersData.length; ++i)
@@ -378,15 +393,16 @@ m.AttackManager.prototype.getEnemyPlayer = function(gameState, attack)
 			if (enemyDefense > 6)
 				veto[i] = true;
 		}
-	} 
+	}
 
-	// then if not a huge attack, continue attacking our previous target as long as it has some entities, 
+	// then if not a huge attack, continue attacking our previous target as long as it has some entities,
 	// otherwise target the most accessible one
 	if (attack.type !== "HugeAttack")
 	{
-		if (attack.targetPlayer === undefined && this.currentEnemyPlayer !== undefined
-			&& !this.defeated[this.currentEnemyPlayer]
-			&& gameState.getEnemyEntities(this.currentEnemyPlayer) > 0)
+		if (attack.targetPlayer === undefined && this.currentEnemyPlayer !== undefined &&
+			!this.defeated[this.currentEnemyPlayer] &&
+			gameState.isPlayerEnemy(this.currentEnemyPlayer) &&
+			gameState.getEntities(this.currentEnemyPlayer).hasEntities())
 			return this.currentEnemyPlayer;
 
 		let distmin;
@@ -425,7 +441,7 @@ m.AttackManager.prototype.getEnemyPlayer = function(gameState, attack)
 
 	// then let's target our strongest enemy (basically counting enemies units)
 	// with priority to enemies with civ center
-	var max = 0;
+	let max = 0;
 	for (let i = 1; i < gameState.sharedScript.playersData.length; ++i)
 	{
 		if (veto[i])
@@ -434,7 +450,7 @@ m.AttackManager.prototype.getEnemyPlayer = function(gameState, attack)
 			continue;
 		let enemyCount = 0;
 		let enemyCivCentre = false;
-		for (let ent of gameState.getEnemyEntities(i).values())
+		for (let ent of gameState.getEntities(i).values())
 		{
 			enemyCount++;
 			if (ent.hasClass("CivCentre"))
@@ -452,6 +468,103 @@ m.AttackManager.prototype.getEnemyPlayer = function(gameState, attack)
 	return enemyPlayer;
 };
 
+/** f.e. if we have changed diplomacy with another player. */
+m.AttackManager.prototype.cancelAttacksAgainstPlayer = function(gameState, player)
+{
+	for (let attackType in this.upcomingAttacks)
+		for (let attack of this.upcomingAttacks[attackType])
+			if (attack.targetPlayer === player)
+				attack.targetPlayer = undefined;
+
+	for (let attackType in this.startedAttacks)
+		for (let i = 0; i < this.startedAttacks[attackType].length; ++i)
+		{
+			let attack = this.startedAttacks[attackType][i];
+			if (attack.targetPlayer === player)
+			{
+				attack.Abort(gameState);
+				this.startedAttacks[attackType].splice(i--, 1);
+			}
+		}
+};
+
+m.AttackManager.prototype.raidTargetEntity = function(gameState, ent)
+{
+	let data = { "target": ent };
+	let attackPlan = new m.AttackPlan(gameState, this.Config, this.totalNumber, "Raid", data);
+	if (!attackPlan.failed)
+	{
+		if (this.Config.debug > 1)
+			API3.warn("Headquarters: Raiding plan " + this.totalNumber);
+		this.totalNumber++;
+		attackPlan.init(gameState);
+		this.upcomingAttacks.Raid.push(attackPlan);
+	}
+	this.raidNumber++;
+};
+
+/**
+ * Switch defense armies into an attack one against the given target
+ * data.range: transform all defense armies inside range of the target into a new attack
+ * data.armyID: transform only the defense army ID into a new attack
+ * data.uniqueTarget: the attack will stop when the target is destroyed or captured
+ */
+m.AttackManager.prototype.switchDefenseToAttack = function(gameState, target, data)
+{
+	if (!target || !target.position())
+		return false;
+	if (!data.range && !data.armyID)
+	{
+		API3.warn(" attackManager.switchDefenseToAttack inconsistent data " + uneval(data));
+		return false;
+	}
+	let attackData = data.uniqueTarget ? { "uniqueTargetId": target.id() } : undefined;
+	let pos = target.position();
+	let attackType = "Attack";
+	let attackPlan = new m.AttackPlan(gameState, this.Config, this.totalNumber, attackType, attackData);
+	if (attackPlan.failed)
+		return false;
+	this.totalNumber++;
+	attackPlan.init(gameState);
+	this.startedAttacks[attackType].push(attackPlan);
+
+	for (let army of gameState.ai.HQ.defenseManager.armies)
+	{
+		if (data.range)
+		{
+			army.recalculatePosition(gameState);
+			if (API3.SquareVectorDistance(pos, army.foePosition) > data.range * data.range)
+				continue;
+		}
+		else if (army.ID != +data.armyID)
+			continue;
+
+		while (army.foeEntities.length > 0)
+			army.removeFoe(gameState, army.foeEntities[0]);
+		while (army.ownEntities.length > 0)
+		{
+			let unitId = army.ownEntities[0];
+			army.removeOwn(gameState, unitId);
+			let unit = gameState.getEntityById(unitId);
+			if (unit && attackPlan.isAvailableUnit(gameState, unit))
+			{
+				unit.setMetadata(PlayerID, "plan", attackPlan.name);
+				attackPlan.unitCollection.updateEnt(unit);
+			}
+		}
+	}
+	if (!attackPlan.unitCollection.hasEntities())
+	{
+		attackPlan.Abort(gameState);
+		return false;
+	}
+	attackPlan.targetPlayer = target.owner();
+	attackPlan.targetPos = pos;
+	attackPlan.target = target;
+	attackPlan.state = "arrived";
+	return true;
+};
+
 m.AttackManager.prototype.Serialize = function()
 {
 	let properties = {
@@ -496,7 +609,7 @@ m.AttackManager.prototype.Deserialize = function(gameState, data)
 		this.upcomingAttacks[key] = [];
 		for (let dataAttack of data.upcomingAttacks[key])
 		{
-			let attack =  new m.AttackPlan(gameState, this.Config, 0);
+			let attack = new m.AttackPlan(gameState, this.Config, dataAttack.properties.name);
 			attack.Deserialize(gameState, dataAttack);
 			attack.init(gameState);
 			this.upcomingAttacks[key].push(attack);
@@ -509,7 +622,7 @@ m.AttackManager.prototype.Deserialize = function(gameState, data)
 		this.startedAttacks[key] = [];
 		for (let dataAttack of data.startedAttacks[key])
 		{
-			let attack =  new m.AttackPlan(gameState, this.Config, 0);
+			let attack = new m.AttackPlan(gameState, this.Config, dataAttack.properties.name);
 			attack.Deserialize(gameState, dataAttack);
 			attack.init(gameState);
 			this.startedAttacks[key].push(attack);
