@@ -1,11 +1,11 @@
 function TechnologyManager() {}
 
 TechnologyManager.prototype.Schema =
-	"<a:component type='system'/><empty/>";
+	"<empty/>";
 
 TechnologyManager.prototype.Serialize = function()
 {
-	// The modifications cache will be affected by property reads from the GUI and other places so we shouldn't 
+	// The modifications cache will be affected by property reads from the GUI and other places so we shouldn't
 	// serialize it.
 
 	var ret = {};
@@ -25,8 +25,8 @@ TechnologyManager.prototype.Init = function()
 	this.researchStarted = {}; // technologies which are being researched currently (non-queued)
 
 	// This stores the modifications to unit stats from researched technologies
-	// Example data: {"ResourceGatherer/Rates/food.grain": [ 
-	//                     {"multiply": 1.15, "affects": ["Female", "Infantry Swordsman"]},
+	// Example data: {"ResourceGatherer/Rates/food.grain": [
+	//                     {"multiply": 1.15, "affects": ["FemaleCitizen", "Infantry Swordsman"]},
 	//                     {"add": 2}
 	//                 ]}
 	this.modifications = {};
@@ -34,15 +34,14 @@ TechnologyManager.prototype.Init = function()
 	                             // e.g. { "Attack/Melee/Hack" : {5: {"origValue": 8, "newValue": 10}, 7: {"origValue": 9, "newValue": 12}, ...}, ...}
 	                             // where 5 and 7 are entity id's
 
-	this.typeCounts = {}; // stores the number of entities of each type 
 	this.classCounts = {}; // stores the number of entities of each Class
 	this.typeCountsByClass = {}; // stores the number of entities of each type for each class i.e.
 	                             // {"someClass": {"unit/spearman": 2, "unit/cav": 5} "someOtherClass":...}
 
-	// Some technologies are automatically researched when their conditions are met.  They have no cost and are 
+	// Some technologies are automatically researched when their conditions are met.  They have no cost and are
 	// researched instantly.  This allows civ bonuses and more complicated technologies.
 	this.autoResearchTech = {};
-	var allTechs = Engine.QueryInterface(SYSTEM_ENTITY, IID_TechnologyTemplateManager).GetAllTechs();
+	var allTechs = Engine.QueryInterface(SYSTEM_ENTITY, IID_DataTemplateManager).GetAllTechs();
 	for (var key in allTechs)
 	{
 		if (allTechs[key].autoResearch || allTechs[key].top)
@@ -50,7 +49,7 @@ TechnologyManager.prototype.Init = function()
 	}
 };
 
-TechnologyManager.prototype.OnUpdate = function() 
+TechnologyManager.prototype.OnUpdate = function()
 {
 	this.UpdateAutoResearch();
 };
@@ -59,10 +58,10 @@ TechnologyManager.prototype.OnUpdate = function()
 // This function checks if the requirements of any autoresearch techs are met and if they are it researches them
 TechnologyManager.prototype.UpdateAutoResearch = function()
 {
-	var cmpTechTempMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_TechnologyTemplateManager);
+	var cmpDataTempMan = Engine.QueryInterface(SYSTEM_ENTITY, IID_DataTemplateManager);
 	for (var key in this.autoResearchTech)
 	{
-		var tech = cmpTechTempMan.GetTemplate(key);
+		var tech = cmpDataTempMan.GetTechnologyTemplate(key);
 		if ((tech.autoResearch && this.CanResearch(key))
 			|| (tech.top && (this.IsTechnologyResearched(tech.top) || this.IsTechnologyResearched(tech.bottom))))
 		{
@@ -75,7 +74,7 @@ TechnologyManager.prototype.UpdateAutoResearch = function()
 
 TechnologyManager.prototype.GetTechnologyTemplate = function(tech)
 {
-	return Engine.QueryInterface(SYSTEM_ENTITY, IID_TechnologyTemplateManager).GetTemplate(tech);
+	return Engine.QueryInterface(SYSTEM_ENTITY, IID_DataTemplateManager).GetTechnologyTemplate(tech);
 };
 
 // Checks an entity template to see if its technology requirements have been met
@@ -86,8 +85,8 @@ TechnologyManager.prototype.CanProduce = function (templateName)
 
 	if (template.Identity && template.Identity.RequiredTechnology)
 		return this.IsTechnologyResearched(template.Identity.RequiredTechnology);
-	else
-		return true; // If there is no required technology then this entity can be produced
+	// If there is no required technology then this entity can be produced
+	return true;
 };
 
 TechnologyManager.prototype.IsTechnologyResearched = function(tech)
@@ -98,17 +97,14 @@ TechnologyManager.prototype.IsTechnologyResearched = function(tech)
 // Checks the requirements for a technology to see if it can be researched at the current time
 TechnologyManager.prototype.CanResearch = function(tech)
 {
-	var template = this.GetTechnologyTemplate(tech);
+	let template = this.GetTechnologyTemplate(tech);
+
 	if (!template)
 	{
 		warn("Technology \"" + tech + "\" does not exist");
 		return false;
 	}
 
-	// The technology which this technology supersedes is required
-	if (template.supersedes && !this.IsTechnologyResearched(template.supersedes))
-		return false;
-
 	if (template.top && this.IsInProgress(template.top) ||
 	    template.bottom && this.IsInProgress(template.bottom))
 		return false;
@@ -122,81 +118,67 @@ TechnologyManager.prototype.CanResearch = function(tech)
 	if (this.IsTechnologyResearched(tech))
 		return false;
 
-	return this.CheckTechnologyRequirements(template.requirements || null);
+	return this.CheckTechnologyRequirements(DeriveTechnologyRequirements(template, Engine.QueryInterface(this.entity, IID_Player).GetCiv()));
 };
 
-// Private function for checking a set of requirements is met
-TechnologyManager.prototype.CheckTechnologyRequirements = function(reqs)
+/**
+ * Private function for checking a set of requirements is met
+ * @param {object} reqs - Technology requirements as derived from the technology template by globalscripts
+ * @param {boolean} civonly - True if only the civ requirement is to be checked
+ *
+ * @return true if the requirements pass, false otherwise
+ */
+TechnologyManager.prototype.CheckTechnologyRequirements = function(reqs, civonly = false)
 {
-	// If there are no requirements then all requirements are met
+	let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+
 	if (!reqs)
-		return true;
-
-	if (reqs.tech)
-	{
-		return this.IsTechnologyResearched(reqs.tech);
-	}
-	else if (reqs.all)
-	{
-		for (var i = 0; i < reqs.all.length; i++)
-		{
-			if (!this.CheckTechnologyRequirements(reqs.all[i]))
-				return false;
-		}
-		return true;
-	}
-	else if (reqs.any)
-	{
-		for (var i = 0; i < reqs.any.length; i++)
-		{
-			if (this.CheckTechnologyRequirements(reqs.any[i]))
-				return true;
-		}
 		return false;
-	}
-	else if (reqs.class)
-	{
-		if (reqs.numberOfTypes)
-		{
-			if (this.typeCountsByClass[reqs.class])
-				return (reqs.numberOfTypes <= Object.keys(this.typeCountsByClass[reqs.class]).length);
-			else
-				return false;
-		}
-		else if (reqs.number)
-		{
-			if (this.classCounts[reqs.class])
-				return (reqs.number <= this.classCounts[reqs.class]);
-			else
-				return false;
-		}
-	}
-	else if (reqs.civ) 
-	{
-		var cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
-		if (cmpPlayer && cmpPlayer.GetCiv() == reqs.civ) 
-			return true;
-		else 
-			return false;
-	}
 
-	// The technologies requirements are not a recognised format
-	error("Bad requirements " + uneval(reqs));
-	return false;
+	if (civonly || !reqs.length)
+		return true;
+
+	return reqs.some(req => {
+		return Object.keys(req).every(type => {
+			switch (type)
+			{
+			case "techs":
+				return req[type].every(this.IsTechnologyResearched, this);
+
+			case "entities":
+				return req[type].every(this.DoesEntitySpecPass, this);
+			}
+			return false;
+		});
+	});
+};
+
+TechnologyManager.prototype.DoesEntitySpecPass = function(entity)
+{
+	switch (entity.check)
+	{
+	case "count":
+		if (!this.classCounts[entity.class] || this.classCounts[entity.class] < entity.number)
+			return false;
+		break;
+
+	case "variants":
+		if (!this.typeCountsByClass[entity.class] || Object.keys(this.typeCountsByClass[entity.class]).length < entity.number)
+			return false;
+		break;
+	}
+	return true;
 };
 
 TechnologyManager.prototype.OnGlobalOwnershipChanged = function(msg)
 {
-	// This automatically updates typeCounts, classCounts and typeCountsByClass
+	// This automatically updates classCounts and typeCountsByClass
 	var playerID = (Engine.QueryInterface(this.entity, IID_Player)).GetPlayerID();
 	if (msg.to == playerID)
 	{
 		var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
 		var template = cmpTemplateManager.GetCurrentTemplateName(msg.entity);
 
-		this.typeCounts[template] = this.typeCounts[template] || 0;
-		this.typeCounts[template] += 1;
-
 		var cmpIdentity = Engine.QueryInterface(msg.entity, IID_Identity);
 		if (!cmpIdentity)
 			return;
@@ -245,10 +227,6 @@ TechnologyManager.prototype.OnGlobalOwnershipChanged = function(msg)
 	{
 		var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
 		var template = cmpTemplateManager.GetCurrentTemplateName(msg.entity);
-		
-		this.typeCounts[template] -= 1;
-		if (this.typeCounts[template] <= 0)
-			delete this.typeCounts[template];
 
 		// don't use foundations for the class counts
 		if (!Engine.QueryInterface(msg.entity, IID_Foundation))
@@ -277,7 +255,7 @@ TechnologyManager.prototype.OnGlobalOwnershipChanged = function(msg)
 // Marks a technology as researched.  Note that this does not verify that the requirements are met.
 TechnologyManager.prototype.ResearchTechnology = function(tech)
 {
-	this.StoppedResearch(tech); // The tech is no longer being currently researched
+	this.StoppedResearch(tech, false);
 
 	var template = this.GetTechnologyTemplate(tech);
 
@@ -292,54 +270,24 @@ TechnologyManager.prototype.ResearchTechnology = function(tech)
 	// store the modifications in an easy to access structure
 	if (template.modifications)
 	{
-		var affects = [];
-		if (template.affects && template.affects.length > 0)
+		let derivedModifiers = DeriveModificationsFromTech(template);
+		for (let modifierPath in derivedModifiers)
 		{
-			for (let affect of template.affects)
-			{
-				// Put the list of classes into an array for convenient access
-				affects.push(affect.split(/\s+/));
-			}
-		}
-		else
-		{
-			affects.push([]);
-		}
+			if (!this.modifications[modifierPath])
+				this.modifications[modifierPath] = [];
+			this.modifications[modifierPath] = this.modifications[modifierPath].concat(derivedModifiers[modifierPath]);
 
-		// We add an item to this.modifications for every modification in the template.modifications array
-		for (var i in template.modifications)
-		{
-			var modification = template.modifications[i];
-			if (!this.modifications[modification.value])
-				this.modifications[modification.value] = [];
-			
-			var modAffects = affects.slice();
-			if (modification.affects)
-			{
-				var extraAffects = modification.affects.split(/\s+/);
-				for (var a in modAffects)
-					modAffects[a] = modAffects[a].concat(extraAffects);
-			}
-
-			var mod = {"affects": modAffects};
-
-			// copy the modification data into our new data structure
-			for (var j in modification)
-				if (j !== "value" && j !== "affects")
-					mod[j] = modification[j];
-
-			this.modifications[modification.value].push(mod);
-			var component = modification.value.split("/")[0];
+			let component = modifierPath.split("/")[0];
 			if (!modifiedComponents[component])
 				modifiedComponents[component] = [];
-			modifiedComponents[component].push(modification.value);
-			this.modificationCache[modification.value] = {};
+			modifiedComponents[component].push(modifierPath);
+			this.modificationCache[modifierPath] = {};
 		}
 	}
 
 	if (template.replaces && template.replaces.length > 0)
 	{
-		for (var i of template.replaces) 
+		for (var i of template.replaces)
 		{
 			if (!i || this.IsTechnologyResearched(i))
 				continue;
@@ -367,13 +315,14 @@ TechnologyManager.prototype.ResearchTechnology = function(tech)
 	var playerID = cmpPlayer.GetPlayerID();
 	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
 	var ents = cmpRangeManager.GetEntitiesByPlayer(playerID);
+	ents.push(this.entity);
 
 	// Change the EntityLimit if any
 	var cmpPlayerEntityLimits = QueryPlayerIDInterface(playerID, IID_EntityLimits);
 	if (cmpPlayerEntityLimits)
 		cmpPlayerEntityLimits.UpdateLimitsFromTech(tech);
 
-	// always send research finished message 
+	// always send research finished message
 	Engine.PostMessage(this.entity, MT_ResearchFinished, {"player": playerID, "tech": tech});
 
 	for (var component in modifiedComponents)
@@ -381,6 +330,17 @@ TechnologyManager.prototype.ResearchTechnology = function(tech)
 		Engine.PostMessage(SYSTEM_ENTITY, MT_TemplateModification, { "player": playerID, "component": component, "valueNames": modifiedComponents[component]});
 		Engine.BroadcastMessage(MT_ValueModification, { "entities": ents, "component": component, "valueNames": modifiedComponents[component]});
 	}
+
+	if (tech.startsWith("phase") && !template.autoResearch)
+	{
+		let cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
+		cmpGUIInterface.PushNotification({
+			"type": "phase",
+			"players": [playerID],
+			"phaseName": tech,
+			"phaseState": "completed"
+		});
+	}
 };
 
 // Clears the cached data for an entity from the modifications cache
@@ -402,13 +362,13 @@ TechnologyManager.prototype.ApplyModifications = function(valueName, curValue, e
 
 	if (!this.modificationCache[valueName][ent] || this.modificationCache[valueName][ent].origValue != curValue)
 	{
-		this.modificationCache[valueName][ent] = {"origValue": curValue};
-		var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
-		var templateName = cmpTemplateManager.GetCurrentTemplateName(ent);
-		// Ensure that preview or construction entites have the same properties as the final building
-		if (templateName.indexOf("preview|") > -1 || templateName.indexOf("construction|") > -1 )
-			templateName = templateName.slice(templateName.indexOf("|") + 1);
-		this.modificationCache[valueName][ent].newValue = GetTechModifiedProperty(this.modifications, cmpTemplateManager.GetTemplate(templateName), valueName, curValue);
+		let cmpIdentity = Engine.QueryInterface(ent, IID_Identity);
+		if (!cmpIdentity)
+			return curValue;
+		this.modificationCache[valueName][ent] = {
+			"origValue": curValue,
+			"newValue": GetTechModifiedProperty(this.modifications, cmpIdentity.GetClassesList(), valueName, curValue)
+		};
 	}
 
 	return this.modificationCache[valueName][ent].newValue;
@@ -417,7 +377,9 @@ TechnologyManager.prototype.ApplyModifications = function(valueName, curValue, e
 // Alternative version of ApplyModifications, applies to templates instead of entities
 TechnologyManager.prototype.ApplyModificationsTemplate = function(valueName, curValue, template)
 {
-	return GetTechModifiedProperty(this.modifications, template, valueName, curValue);
+	if (!template || !template.Identity)
+		return curValue;
+	return GetTechModifiedProperty(this.modifications, GetIdentityClasses(template.Identity), valueName, curValue);
 };
 
 // Marks a technology as being queued for research
@@ -427,14 +389,38 @@ TechnologyManager.prototype.QueuedResearch = function(tech, researcher)
 };
 
 // Marks a technology as actively being researched
-TechnologyManager.prototype.StartedResearch = function(tech)
+TechnologyManager.prototype.StartedResearch = function(tech, notification)
 {
 	this.researchStarted[tech] = true;
+
+	if (notification && tech.startsWith("phase"))
+	{
+		let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+		let cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
+		cmpGuiInterface.PushNotification({
+			"type": "phase",
+			"players": [cmpPlayer.GetPlayerID()],
+			"phaseName": tech,
+			"phaseState": "started"
+		});
+	}
 };
 
 // Marks a technology as not being currently researched
-TechnologyManager.prototype.StoppedResearch = function(tech)
+TechnologyManager.prototype.StoppedResearch = function(tech, notification)
 {
+	if (notification && tech.startsWith("phase") && this.researchStarted[tech])
+	{
+		let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+		let cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
+		cmpGUIInterface.PushNotification({
+			"type": "phase",
+			"players": [cmpPlayer.GetPlayerID()],
+			"phaseName": tech,
+			"phaseState": "aborted"
+		});
+	}
+
 	delete this.researchQueued[tech];
 	delete this.researchStarted[tech];
 };
@@ -449,7 +435,7 @@ TechnologyManager.prototype.IsInProgress = function(tech)
 };
 
 // Get all techs that are currently being researched
-TechnologyManager.prototype.GetTechsStarted = function()
+TechnologyManager.prototype.GetStartedTechs = function()
 {
 	return this.researchStarted;
 };
@@ -473,10 +459,6 @@ TechnologyManager.prototype.GetQueuedResearch = function()
 {
 	return this.researchQueued;
 };
-TechnologyManager.prototype.GetStartedResearch = function()
-{
-	return this.researchStarted;
-};
 TechnologyManager.prototype.GetResearchedTechs = function()
 {
 	return this.researchedTechs;
