@@ -1,5 +1,21 @@
 // The number of currently visible buttons (used to optimise showing/hiding)
-var g_unitPanelButtons = {"Selection": 0, "Queue": 0, "Formation": 0, "Garrison": 0, "Training": 0, "Research": 0, "Alert": 0, "Barter": 0, "Construction": 0, "Command": 0, "AllyCommand": 0, "Stance": 0, "Gate": 0, "Pack": 0};
+var g_unitPanelButtons = {
+	"Selection": 0,
+	"Queue": 0,
+	"Formation": 0,
+	"Garrison": 0,
+	"Training": 0,
+	"Research": 0,
+	"Alert": 0,
+	"Barter": 0,
+	"Construction": 0,
+	"Command": 0,
+	"AllyCommand": 0,
+	"Stance": 0,
+	"Gate":0,
+	"Pack": 0,
+	"Upgrade": 0
+};
 
 /**
  * Set the position of a panel object according to the index,
@@ -28,74 +44,46 @@ function setPanelObjectPosition(object, index, rowLength, vMargin = 1, hMargin =
  * (i.e. panels with rows of icons) for the currently selected unit.
  *
  * @param guiName Short identifier string of this panel. See g_SelectionPanels.
- * @param unitEntState Entity state of the (first) selected unit.
- * @param payerState Player state
+ * @param unitEntStates Entity states of the selected units
+ * @param playerState Player state
  */
-function setupUnitPanel(guiName, unitEntState, playerState)
+function setupUnitPanel(guiName, unitEntStates, playerState)
 {
 	if (!g_SelectionPanels[guiName])
 	{
 		error("unknown guiName used '" + guiName + "'");
 		return;
 	}
-	var selection = g_Selection.toList();
 
-	var items = g_SelectionPanels[guiName].getItems(unitEntState, selection);
+	let selection = g_Selection.toList();
+	let items = g_SelectionPanels[guiName].getItems(unitEntStates);
 
 	if (!items || !items.length)
 		return;
 
-	var numberOfItems = items.length;
-	var garrisonGroups = new EntityGroups();
-
-	// Determine how many buttons there should be
-	var maxNumberOfItems = g_SelectionPanels[guiName].getMaxNumberOfItems();
-	if (maxNumberOfItems < numberOfItems)
-		numberOfItems = maxNumberOfItems;
-
-	if (g_SelectionPanels[guiName].rowLength)
-		var rowLength = g_SelectionPanels[guiName].rowLength;
-	else
-		var rowLength = 8;
+	let numberOfItems = Math.min(items.length, g_SelectionPanels[guiName].getMaxNumberOfItems());
+	let rowLength = g_SelectionPanels[guiName].rowLength || 8;
 
 	if (g_SelectionPanels[guiName].resizePanel)
 		g_SelectionPanels[guiName].resizePanel(numberOfItems, rowLength);
 
-	// Make buttons
-	for (var i = 0; i < numberOfItems; i++)
+	for (let i = 0; i < numberOfItems; ++i)
 	{
-		var item = items[i];
-
-		// If a tech has been researched it leaves an empty slot
-		if (!item)
-		{
-			if (g_SelectionPanels[guiName].hideItem)
-				g_SelectionPanels[guiName].hideItem(i, rowLength);
-			continue;
-		}
-
-		// STANDARD DATA
-		// add standard data 
-		var data = {
+		let data = {
 			"i": i,
-			"item": item,
-			"selection": selection,
+			"item": items[i],
 			"playerState": playerState,
-			"unitEntState": unitEntState,
+			"player": unitEntStates[0].player,
+			"unitEntStates": unitEntStates,
 			"rowLength": rowLength,
 			"numberOfItems": numberOfItems,
+			// depending on the XML, some of the GUI objects may be undefined
+			"button": Engine.GetGUIObjectByName("unit" + guiName + "Button[" + i + "]"),
+			"icon": Engine.GetGUIObjectByName("unit" + guiName + "Icon[" + i + "]"),
+			"guiSelection": Engine.GetGUIObjectByName("unit" + guiName + "Selection[" + i + "]"),
+			"countDisplay": Engine.GetGUIObjectByName("unit" + guiName + "Count[" + i + "]")
 		};
 
-		// add standard gui objects to the data
-		// depending on the actual XML, some of this may be undefined
-		data.button = Engine.GetGUIObjectByName("unit"+guiName+"Button["+i+"]");
-		data.affordableMask = Engine.GetGUIObjectByName("unit"+guiName+"Unaffordable["+i+"]");
-		data.icon = Engine.GetGUIObjectByName("unit"+guiName+"Icon["+i+"]");
-		data.guiSelection = Engine.GetGUIObjectByName("unit"+guiName+"Selection["+i+"]");
-		data.countDisplay = Engine.GetGUIObjectByName("unit"+guiName+"Count["+i+"]");
-
-
-		// DEFAULTS
 		if (data.button)
 		{
 			data.button.hidden = false;
@@ -104,110 +92,72 @@ function setupUnitPanel(guiName, unitEntState, playerState)
 			data.button.caption = "";
 		}
 
-		if (data.affordableMask)
-			data.affordableMask.hidden = true;	// actually used for the red "lack of resource" overlay, and darkening if unavailable. Sort of a hack.
-
-		// GENERAL DATA
-		// add general data, and a chance to abort on faulty data
-		if (g_SelectionPanels[guiName].addData)
-		{
-			var success = g_SelectionPanels[guiName].addData(data);
-			if (!success)
-				continue; // ignore faulty data
-		}
-
-		// SET CONTENT
-		// run all content setters
-		for (var f in g_SelectionPanels[guiName])
-		{
-			if (f.match(/^set/))
-				g_SelectionPanels[guiName][f](data);
-		}
-
-		// Special case: position
-		if (!g_SelectionPanels[guiName].setPosition)
-			setPanelObjectPosition(data.button, i, rowLength);
+		if (g_SelectionPanels[guiName].setupButton &&
+		    !g_SelectionPanels[guiName].setupButton(data))
+			continue;
 
 		// TODO: we should require all entities to have icons, so this case never occurs
 		if (data.icon && !data.icon.sprite)
-			data.icon.sprite = "bkFillBlack";
-		
+			data.icon.sprite = "BackgroundBlack";
 	}
 
 	// Hide any buttons we're no longer using
-	for (var i = numberOfItems; i < g_unitPanelButtons[guiName]; ++i)
+	for (let i = numberOfItems; i < g_unitPanelButtons[guiName]; ++i)
 		if (g_SelectionPanels[guiName].hideItem)
 			g_SelectionPanels[guiName].hideItem(i, rowLength);
 		else
 			Engine.GetGUIObjectByName("unit"+guiName+"Button["+i+"]").hidden = true;
 
-	// remember the number of items
 	g_unitPanelButtons[guiName] = numberOfItems;
 	g_SelectionPanels[guiName].used = true;
 }
 
-function resourcesToAlphaMask(neededResources)
-{
-	var totalCost = 0;
-	for each (var resource in neededResources)
-		totalCost += resource;
-	var alpha = 50 + Math.round(totalCost/10);
-	alpha = alpha > 125 ? 125 : alpha;
-	return "color: 255 0 0 " + (alpha);
-}
-
 /**
- * Updates the selection panels where buttons are supposed to 
+ * Updates the selection panels where buttons are supposed to
  * depend on the context.
  * Runs in the main session loop via updateSelectionDetails().
  * Delegates to setupUnitPanel to set up individual subpanels,
  * appropriately activated depending on the selected unit's state.
  *
- * @param entState Entity state of the (first) selected unit.
- * @param supplementalDetailsPanel Reference to the 
+ * @param entStates Entity states of the selected units
+ * @param supplementalDetailsPanel Reference to the
  *        "supplementalSelectionDetails" GUI Object
  * @param commandsPanel Reference to the "commandsPanel" GUI Object
- * @param selection Array of currently selected entity IDs.
  */
-function updateUnitCommands(entState, supplementalDetailsPanel, commandsPanel, selection)
+function updateUnitCommands(entStates, supplementalDetailsPanel, commandsPanel)
 {
-	for each (var panel in g_SelectionPanels)
-		panel.used = false;
+	for (let panel in g_SelectionPanels)
+		g_SelectionPanels[panel].used = false;
 
 	// If the selection is friendly units, add the command panels
-	var player = Engine.GetPlayerID();
 
 	// Get player state to check some constraints
 	// e.g. presence of a hero or build limits
-	var simState = GetSimState();
-	var playerState = simState.players[player];
+	let playerStates = GetSimState().players;
+	let playerState = playerStates[Engine.GetPlayerID()];
 
-	if (entState.player == player || g_DevSettings.controlAll)
+	if (g_IsObserver || entStates.every(entState => controlsPlayer(entState.player)))
 	{
 		for (var guiName of g_PanelsOrder)
 		{
-			if (
-				g_SelectionPanels[guiName].conflictsWith && 
-				g_SelectionPanels[guiName].conflictsWith.some(function (p) { return g_SelectionPanels[p].used; })
-			)
+			if (g_SelectionPanels[guiName].conflictsWith &&
+			    g_SelectionPanels[guiName].conflictsWith.some(p => g_SelectionPanels[p].used))
 				continue;
 
-			setupUnitPanel(guiName, entState, playerState);
+			setupUnitPanel(guiName, entStates, playerStates[entStates[0].player]);
 		}
 
 		supplementalDetailsPanel.hidden = false;
 		commandsPanel.hidden = false;
 	}
-	else if (playerState.isMutualAlly[entState.player]) // owned by allied player
+	else if (playerState.isMutualAlly[entStates[0].player]) // owned by allied player
 	{
 		// TODO if there's a second panel needed for a different player
 		// we should consider adding the players list to g_SelectionPanels
-		setupUnitPanel("Garrison", entState, playerState);
-		setupUnitPanel("AllyCommand", entState, playerState);
-		if (g_SelectionPanels["Garrison"].used)
-			supplementalDetailsPanel.hidden = false;
-		else
-			supplementalDetailsPanel.hidden = true;
+		setupUnitPanel("Garrison", entStates, playerState);
+		setupUnitPanel("AllyCommand", entStates, playerState);
+
+		supplementalDetailsPanel.hidden = !g_SelectionPanels.Garrison.used;
 
 		commandsPanel.hidden = true;
 	}
@@ -271,13 +221,10 @@ function getAllBuildableEntitiesFromSelection()
 function getNumberOfRightPanelButtons()
 {
 	var sum = 0;
-	if (g_SelectionPanels["Construction"].used)
-		sum += g_unitPanelButtons["Construction"];
-	if (g_SelectionPanels["Training"].used)
-		sum += g_unitPanelButtons["Training"];
-	if (g_SelectionPanels["Pack"].used)
-		sum += g_unitPanelButtons["Pack"];
-	if (g_SelectionPanels["Gate"].used)
-		sum += g_unitPanelButtons["Gate"];
+
+	for (let prop of ["Construction", "Training", "Pack", "Gate", "Upgrade"])
+		if (g_SelectionPanels[prop].used)
+			sum += g_unitPanelButtons[prop];
+
 	return sum;
 }
